/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.control.activity.task;

/**
 * Interface representing a task.
 */
public interface Task
{

  /**
   * The commonly used nam to refer to this interface.
   */
  String  ITF_NAME       = "task";

  /**
   * An integer that can be used with some schedulers to specify that the task
   * should be executed again.
   */
  Integer EXECUTE_AGAIN  = new Integer(0);

  /**
   * An integer that can be used with some schedulers to specify that the task
   * should not be executed again.
   */
  Integer STOP_EXECUTING = new Integer(-1);

  /**
   * The method that is executed.
   * 
   * @param hints execution paramters.
   * @return an object representing the execution result.
   * @throws InterruptedException if this task is interrupted.
   */
  Object execute(Object hints) throws InterruptedException;

  /**
   * Call back method used by scheduler to notify a task that has been
   * interrupted. Task implementation may override this method to free resources
   * and ensure that the task execution will end.
   */
  void interrupted();

  /**
   * Call back method to notify that the task as been correctly registered in
   * the task manager.
   * 
   * @param controlItf An interface that can be used to control the execution of
   *          the task. The runtime type of this interface depends on the
   *          registration hints. Can be <code>null</code>.
   */
  void registered(Object controlItf);

  /**
   * Call back method to notify that the task has been unregistered from the
   * activity manager.
   */
  void unregistered();
}