/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.control.lifecycle;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.julia.control.lifecycle.LifeCycleCoordinator;

/**
 * Calls the {@link PrepareStopLifeCycleController#prepareStopFc() }method of
 * the implementation of the component, this controller belong.
 */
public abstract class ContainerPrepareStopLifeCycleControllerMixin
    implements
      LifeCycleCoordinator
{

  /**
   * @see LifeCycleCoordinator#setFcStopping(LifeCycleCoordinator)
   */
  public void setFcStopping(LifeCycleCoordinator coordinator)
      throws IllegalLifeCycleException
  {
    Object content = null;
    try
    {
      content = _this_weaveableOptC.getFcInterface("/content");
    }
    catch (NullPointerException e)
    {
    }
    catch (NoSuchInterfaceException e)
    {
    }
    if (content != null && content instanceof PrepareStopLifeCycleController)
    {
      ((PrepareStopLifeCycleController) content).prepareStopFc();
    }
    _super_setFcStopping(coordinator);
  }

  // -------------------------------------------------------------------------
  // Fields and methods required by the mixin class in the base class
  // -------------------------------------------------------------------------

  /**
   * The <tt>weaveableOptC</tt> field required by this mixin. This field is
   * supposed to reference the {@link Component}interface of the component to
   * which this controller object belongs.
   */
  public Component _this_weaveableOptC;

  /**
   * The
   * {@link LifeCycleCoordinator#setFcStopping(LifeCycleCoordinator) setFcStopping}
   * method overriden by this mixin.
   * 
   * @see LifeCycleCoordinator#setFcStopping(LifeCycleCoordinator)
   */
  public abstract void _super_setFcStopping(LifeCycleCoordinator coordinator)
      throws IllegalLifeCycleException;

}
