/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message;

class BasicChunkFactoryImpl<T extends AbstractChunk<T>>
    extends
      ChunkFactoryReference<T>
{
  private final Class<T> chunkClass;
  private final T        chunkInstance;

  BasicChunkFactoryImpl(final Class<T> chunkClass, final MessageManager owner)
      throws Exception
  {
    super(owner);
    this.chunkClass = chunkClass;
    chunkInstance = chunkClass.newInstance();
  }

  protected T createChunk()
  {
    T newChunk = chunkInstance.newChunk();
    newChunk.initAbstractChunk(this, getOwner());
    return newChunk;
  }

  protected void recycleChunk(T chunk)
  {
    chunk.recycle();
  }

  final Class<T> getChunkClass()
  {
    return chunkClass;
  }
}
