/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message.codec;

import java.io.IOException;

/**
 * Gathers an input and an output used by {@link MessageCodec}.
 */
public interface CodecInputOutput
{
  /**
   * Returns an input that can be used to read data to decode.
   * 
   * @return an input.
   * @throws IOException if the input can't be created.
   */
  Object getInput() throws IOException;

  /**
   * Set the input. This is usefull for {@link MessageCodec }that use
   * specialized input. If this object is used to decode another message, the
   * codec can reuse its specialized input.
   * 
   * @param input the new input.
   */
  void setInput(Object input);

  /**
   * Returns an output that can be used to write encoded data.
   * 
   * @return an output.
   * @throws IOException if the output can't be created.
   */
  Object getOutput() throws IOException;

  /**
   * Set the output. This is usefull for {@link MessageCodec }that use
   * specialized output. If this object is used to decode another message, the
   * codec can reuse its specialized output.
   * 
   * @param output the new output.
   */
  void setOutput(Object output);

}