/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message.codec;

import java.util.Map;

import org.objectweb.fractal.api.Component;

/**
 * TODO refactor codec manager. 
 * Component interface that manage dynamic codecs.
 * Each codec is identified with a codec name.
 */
public interface CodecManager
{
  /** The default name of interface of this type. */
  String ITF_NAME = "codec-manager";

  /**
   * Adds a codec component for the specified codec name. If a codec component
   * for the given codec name is already present, this method do nothing.
   * 
   * @param codecName the name of the codec.
   * @param context additional parameters.
   * @throws CodecManagerException if the manager can't add the codec.
   */
  void addCodec(String codecName, Map<String, Object > context)
      throws CodecManagerException;

  /**
   * Removes a codec component. The codec component must not be in use (i.e. the
   * component is not shared by different composites).
   * 
   * @param codecName the name of the codec.
   * @throws CodecManagerException the the given codec name is unknown, or if
   *           the codec component is in use.
   */
  void removeCodec(String codecName) throws CodecManagerException;

  /**
   * Returns the codec component for the given codec name.
   * 
   * @param codecName the name of the codec.
   * @return the codec component for the given codec name.
   * @throws CodecManagerException the the given codec name is unknown.
   */
  Component getCodec(String codecName) throws CodecManagerException;
}
