/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message.codec;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;

import org.objectweb.dream.message.Message;

/**
 * Codec helper component for codecs using {@link ObjectOutputStream }and
 * {@link ObjectInputStream}. It manages the creation and the reuse of
 * {@link ObjectOutputStream }and {@link ObjectInputStream}. This codec
 * implementation requires a {@link InputStream }as input and a
 * {@link OutputStream }as output.
 */
public class MessageCodecHelperObjectStreamImpl
    extends
      AbstractDelegatingMessageCodecImpl
    implements
      MessageCodecHelperObjectStreamAttributeController
{

  // ---------------------------------------------------------------------------
  // Attribute field
  // ---------------------------------------------------------------------------
  boolean reuseObjectStream;

  // ---------------------------------------------------------------------------
  // Implementation of the MessageCodec interface
  // ---------------------------------------------------------------------------

  /**
   * @see MessageCodec#encode(CodecInputOutput, Message)
   */
  public void encode(CodecInputOutput cio, Message message)
      throws IOException
  {
    Object output = cio.getOutput();
    ObjectOutputStream oos;
    if (output instanceof ObjectOutputStream)
    {
      oos = (ObjectOutputStream) output;
    }
    else
    {
      if (!(output instanceof OutputStream))
      {
        throw new IOException("Unknown output : " + output);
      }
      oos = new ObjectOutputStream((OutputStream) output);
      cio.setOutput(oos);
    }
    delegateMessageCodecItf.encode(cio, message);
    oos.flush();
    if (!reuseObjectStream)
    {
      cio.setOutput(output);
    }
  }

  /**
   * @see MessageCodec#decode(CodecInputOutput)
   */
  public Message decode(CodecInputOutput cio) throws IOException
  {
    Object input = cio.getInput();
    ObjectInputStream ois;
    if (input instanceof ObjectInputStream)
    {
      ois = (ObjectInputStream) input;
    }
    else
    {
      if (!(input instanceof InputStream))
      {
        throw new IOException("Unknown input : " + input);
      }
      ois = new ObjectInputStream((InputStream) input);
      cio.setInput(ois);
    }
    Message message = delegateMessageCodecItf.decode(cio);
    if (!reuseObjectStream)
    {
      cio.setInput(input);
    }
    return message;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the AttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see MessageCodecHelperObjectStreamAttributeController#getReuseObjectStream()
   */
  public boolean getReuseObjectStream()
  {
    return reuseObjectStream;
  }

  /**
   * @see MessageCodecHelperObjectStreamAttributeController#setReuseObjectStream(boolean)
   */
  public void setReuseObjectStream(boolean reuseObjectStream)
  {
    this.reuseObjectStream = reuseObjectStream;
  }

}
