/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.pool;

/**
 * This interface defines an object pool. An object pool aims at improving
 * performances by avoiding unnecessary object creations.
 */
public interface ObjectPool
{
  /** The commonly used name to refer to this interface. */
  String ITF_NAME = "object-pool";

  /**
   * Returns a new instance of object.
   * 
   * @return a new instance of object.
   */
  Recyclable newInstance();

  /**
   * Recycles the object so that it can be reused.
   * 
   * @param recyclable the object to be recycled.
   */
  void recycleInstance(Recyclable recyclable);

}