/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.util;

import java.util.Iterator;

/**
 * An iterator that use a delegate iterator, but does not provides a remove
 * operation.
 * @param <T> the type of the objects this iterator iterates.
 */
public class IteratorNoRemove<T> implements Iterator<T>
{
  private final Iterator<T> delegate;

  /**
   * @param delegate the iterator
   */
  public IteratorNoRemove(Iterator<T> delegate)
  {
    this.delegate = delegate;
  }

  /**
   * @see Iterator#hasNext()
   */
  public boolean hasNext()
  {
    return delegate.hasNext();
  }

  /**
   * @see Iterator#next()
   */
  public T next()
  {
    return delegate.next();
  }

  /**
   * Always throws a {@link UnsupportedOperationException}.
   * 
   * @see java.util.Iterator#remove()
   */
  public void remove()
  {
    throw new UnsupportedOperationException();
  }

}
