/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message.codec;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.HashMap;

import junit.framework.TestCase;

import org.objectweb.dream.adl.FactoryFactory;
import org.objectweb.dream.message.ChunkFactoryReference;
import org.objectweb.dream.message.ChunkTest;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.UtilTest;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;

public class TestMessageCodecObjectStream extends TestCase
{
  private static final String NAME1 = "toto";
  private static final String NAME2 = "titi";

  private Component           testComponent;
  private MessageCodec        messageCodecItf;
  private MessageManager      messageManagerItf;

  private CodecInputOutput    codecInputOutput;

  private Message    message;

  private Message    subMessage1;
  private Message    subMessage2;
  private ChunkTest           chunk1;
  private ChunkTest           chunk2;

  protected void setUp() throws Exception
  {
    testComponent = (Component) FactoryFactory.getFactory().newComponent(
        getClass().getName(), new HashMap());
    Fractal.getLifeCycleController(testComponent).startFc();

    messageCodecItf = (MessageCodec) testComponent
        .getFcInterface(MessageCodec.ITF_NAME);
    messageManagerItf = (MessageManager) testComponent
        .getFcInterface(MessageManager.ITF_NAME);

    codecInputOutput = new CodecInputOutputImpl();

    message = messageManagerItf.createMessage();
    subMessage1 = messageManagerItf.createMessage();
    subMessage2 = messageManagerItf.createMessage();

    ChunkFactoryReference<ChunkTest> f = messageManagerItf
        .getChunkFactory(ChunkTest.class);
    chunk1 = messageManagerItf.createChunk(f);
    chunk2 = messageManagerItf.createChunk(f);
  }

  public void testMessageCodecPrimitive1() throws Exception
  {
    Message decoded = encodeDecode(message);
    UtilTest.assertMessageEquals(decoded, message, messageManagerItf);
  }

  public void testMessageCodecPrimitive2() throws Exception
  {
    messageManagerItf.addChunk(message, NAME1, chunk1);
    Message decoded = encodeDecode(message);
    UtilTest.assertMessageEquals(decoded, message, messageManagerItf);
  }

  public void testMessageCodecPrimitive3() throws Exception
  {
    messageManagerItf.addChunk(message, NAME1, chunk1);
    messageManagerItf.addChunk(message, NAME2, chunk2);
    Message decoded = encodeDecode(message);
    UtilTest.assertMessageEquals(decoded, message, messageManagerItf);
  }

  public void testMessageCodecComposite1() throws Exception
  {
    messageManagerItf.addSubMessage(message, subMessage1);
    Message decoded = encodeDecode(message);
    UtilTest.assertMessageEquals(decoded, message, messageManagerItf);
  }

  public void testMessageCodecComposite2() throws Exception
  {
    messageManagerItf.addChunk(subMessage1, NAME1, chunk1);
    messageManagerItf.addSubMessage(message, subMessage1);
    Message decoded = encodeDecode(message);
    UtilTest.assertMessageEquals(decoded, message, messageManagerItf);
  }

  public void testMessageCodecComposite3() throws Exception
  {
    messageManagerItf.addChunk(subMessage1, NAME1, chunk1);
    messageManagerItf.addChunk(subMessage2, NAME2, chunk2);
    messageManagerItf.addSubMessage(subMessage1, subMessage2);
    messageManagerItf.addSubMessage(message, subMessage1);
    Message decoded = encodeDecode(message);
    UtilTest.assertMessageEquals(decoded, message, messageManagerItf);
  }

  private Message encodeDecode(Message message)
      throws Exception
  {
    ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
    codecInputOutput.setOutput(outputStream);
    messageCodecItf.encode(codecInputOutput, message);
    ByteArrayInputStream inputStream = new ByteArrayInputStream(outputStream
        .toByteArray());
    codecInputOutput.setInput(inputStream);
    return messageCodecItf.decode(codecInputOutput);
  }

}
