/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream;

import java.lang.management.GarbageCollectorMXBean;
import java.lang.management.ManagementFactory;
import java.text.DecimalFormat;
import java.text.NumberFormat;

import org.objectweb.fractal.api.Component;

import junit.framework.TestCase;

public abstract class AbstractPerf extends TestCase
{
  protected Component              testComponent;
  protected NumberFormat           groupFormat;
  protected NumberFormat           timeFormat;
  protected NumberFormat           rateFormat;

  protected GarbageCollectorMXBean garbageCollectorMXBean;

  protected void setUp() throws Exception
  {
    groupFormat = new DecimalFormat("###");
    timeFormat = new DecimalFormat("#0.00");
    rateFormat = new DecimalFormat("0.00E0");
    garbageCollectorMXBean = ManagementFactory
        .getGarbageCollectorMXBeans().get(0);
  }

  protected void runPerf(int burstSize, int nbBurst) throws Exception
  {
    long start = System.currentTimeMillis();
    long gcStart = garbageCollectorMXBean.getCollectionTime();
    runPerf1(burstSize, nbBurst);
    double time = (System.currentTimeMillis() - start) / 1000D;
    double gcTime = (garbageCollectorMXBean.getCollectionTime() - gcStart) / 1000D;
    printResult(burstSize, nbBurst, time, gcTime);
  }

  protected abstract void runPerf1(int burstSize, int nbBurst) throws Exception;

  private void printResult(int burstSize, int nbBurst, double time,
      double gcTime)
  {
    StringBuffer sb = new StringBuffer();
    sb.append(burstSize * nbBurst);
    sb.append(" loops; group of ");
    addSpace(3, groupFormat.format(burstSize), sb);
    sb.append(" ; finished in ");
    addSpace(5, timeFormat.format(time), sb);
    sb.append("s; rate= ").append(
        rateFormat.format((burstSize * nbBurst) / time));
    sb.append(" loops/s; gcTime= ");
    sb.append(timeFormat.format(gcTime)).append(" s (");
    sb.append(NumberFormat.getPercentInstance().format(gcTime / time));
    sb.append(").");
    System.out.println(sb.toString());
  }

  private void addSpace(int nbChar, String s, StringBuffer toAppendTo)
  {
    for (int i = 0; i < nbChar - s.length(); i++)
    {
      toAppendTo.append(' ');
    }
    toAppendTo.append(s);
  }

}
