/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.adl.ant;

import java.io.CharArrayReader;
import java.io.CharArrayWriter;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Map;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Reference;
import org.apache.tools.ant.util.ClasspathUtils;
import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.adl.Loader;

/**
 * TODO Add class comment
 */
public class CheckAdlTask extends Task
{
  // class path helper.
  ClasspathUtils.Delegate cpDelegate;

  // adl to check.
  String                  adl;
  File                    adlList;

  Loader                  loaderItf;

  /**
   * @see org.apache.tools.ant.Task#init()
   */
  public void init() throws BuildException
  {
    cpDelegate = ClasspathUtils.getDelegate(this);
    Factory factory = FactoryFactory.getFactory();
    Map loaderComp;
    try
    {
      loaderComp = (Map) factory.newComponent(
          "org.objectweb.fractal.adl.BasicLoader", new HashMap());
    }
    catch (ADLException e)
    {
      throw new BuildException("Unable to create ADL loader", e);
    }
    loaderItf = (Loader) loaderComp.get("loader");
    if (loaderItf == null)
    {
      throw new BuildException("Unable to create ADL loader");
    }
    super.init();
  }

  /**
   * Sets classpathref attribute.
   * 
   * @param r classpath reference.
   */
  public void setClasspathref(Reference r)
  {
    cpDelegate.setClasspathref(r);
  }

  /**
   * Creates nested classpath element.
   * 
   * @return nested classpath element.
   */
  public Path createClasspath()
  {
    return cpDelegate.createClasspath();
  }

  /**
   * Sets the adl to check.
   * 
   * @param adl the adl to check
   */
  public void setAdl(String adl)
  {
    if (adlList != null)
    {
      throw new BuildException("cannot specify both adl and adlList");
    }
    this.adl = adl;
  }

  /**
   * Sets the file name containing the list of adl to check.
   * 
   * @param adlList the file name of the adl list.
   */
  public void setAdllist(File adlList)
  {
    if (adl != null)
    {
      throw new BuildException("cannot specify both adl and adlList");
    }
    this.adlList = adlList;
  }

  /**
   * @see org.apache.tools.ant.Task#execute()
   */
  public void execute()
  {
    if (adl != null)
    {
      check(adl);
    }
    else
    {
      try
      {
        LineNumberReader lineNumberReader = new LineNumberReader(
            new FileReader(adlList));
        String line = lineNumberReader.readLine();
        while (line != null)
        {
          line = line.trim();
          if (line.length() != 0 && !line.startsWith("#"))
          {
            check(line.trim());
          }
          line = lineNumberReader.readLine();
        }
      }
      catch (IOException e)
      {
        throw new BuildException("unable to read adlList file ", e);
      }
    }
  }

  void check(String adl)
  {
    System.out.println("Checking " + adl);
    try
    {
      Map context = new HashMap();
      context.put("classloader", cpDelegate.getClassLoader());
      loaderItf.load(adl, context);
    }
    catch (ADLException e)
    {
      System.err.println("Error in ADL : " + adl);

      try
      {
        CharArrayWriter charArrayWriter = new CharArrayWriter();
        PrintWriter printWriter = new PrintWriter(charArrayWriter, true);
        e.printStackTrace(printWriter);

        LineNumberReader reader = new LineNumberReader(new CharArrayReader(
            charArrayWriter.toCharArray()));
        String line = reader.readLine();
        while (line != null && (!line.trim().startsWith("at")))
        {
          System.err.println(line);
          line = reader.readLine();
        }
      }
      catch (IOException ioe)
      {
        throw new BuildException(ioe);
      }
      throw new BuildException("Error found in the following ADL : " + adl, e);
    }
  }
}
