/**
 * Copyright (C) 2002
 */

package org.objectweb.util.monolog.wrapper.log4j;

import org.apache.log4j.nt.NTEventLogAppender;
import org.apache.log4j.PatternLayout;
import org.objectweb.util.monolog.api.Handler;
import org.objectweb.util.monolog.wrapper.log4j.PatternConverter;
import org.objectweb.util.monolog.api.MonologFactory;
import org.apache.log4j.Priority;


import java.util.HashMap;
import java.util.Map;

/**
 * This class is the wrapper to the org.apache.log4j.nt.NTEventLogAppender
 *
 * @author Sebastien Chassande-Barrioz
 * @author Igor Smirnov
 */
public class NTEventLogHandler extends NTEventLogAppender implements Handler {

	/**
	 * This fields contains the properties of the Handler
	 */
	protected HashMap prop = null;

	public NTEventLogHandler() {
		super();
	}

	/**
	 * It Builds a new NTEventLogHandler. It is needed to specify an handler
	 * type.
	 * @param name is the handler name.
	 */
	public NTEventLogHandler(String name) {
		super();
		setName(name);
		prop = new HashMap();
	}

	public Map getAttributes() {
		return prop;
	}

	public void setAttributes(Map attributes) {
		prop.clear();
		prop.putAll(attributes);
	}

	// IMPLEMENTATION OF THE Handler INTERFACE //
	//---------------------------------------------//

	public String getType() {
		return "ntevent";
	}

	public String[] getAttributeNames() {
		return (String[]) prop.keySet().toArray(new String[0]);
	}

	public Object getAttribute(String key) {
		return prop.get(key);
	}

	public Object setAttribute(String key, Object value) {
		if (prop == null)
			prop = new HashMap();
		Object res = prop.put(key, value);
		if (key.equalsIgnoreCase(Handler.PATTERN_ATTRIBUTE)) {
			setLayout(
			    new PatternLayout(
			        PatternConverter.monolog2log4j((String) value)));
		} else if (key.equalsIgnoreCase("source")) {
			setSource((String) value);
		} else if (key.equalsIgnoreCase("activation")) {
			MonologFactory mf = (MonologFactory) value;
			String level = (String) prop.get(LEVEL_ATTRIBUTE);
			if (level != null && level.length() > 0) {
				int levelVal = org.objectweb.util.monolog.wrapper.common.LevelImpl.evaluate(level, mf);
				setThreshold(Priority.toPriority(levelVal));
			}
			super.activateOptions();
		}
		return res;
	}
}
